﻿using System;

using CenterSpace.NMath.Core;

namespace CenterSpace.NMath.Core.Examples.CSharp
{
  /// <summary>
  /// A .NET example in C# showing how to solve the Klee Minty cube linear 
  /// programming problem using both primal and dual simplex methods and 
  /// various pivoting strategies.
  /// The Klee Minty cube is a unit cube with perturbed corners. Klee and
  /// Minty showed that Dantzig's simplex algorithm can have worst-case
  /// performance with this cube as the feasible region.
  /// </summary>
  public class PrimalDualSimplexExample
  {
    static void Main( string[] args )
    {
      int n = 200;
      var lpProblem = KleeMintyCube( n );
      Console.WriteLine( "Klee Minty Cube LP problem using Primal Simplex:" );
      Console.WriteLine( "---------------------------------------------------" );
      SolveKleeMintyProblemPrimal( lpProblem );

      Console.WriteLine( "\n" );
      Console.WriteLine( "Klee Minty Cube LP problem using Dual Simplex:" );
      Console.WriteLine( "---------------------------------------------------" );
      SolveKleeMintyProblemDual( lpProblem );

      Console.WriteLine();
      Console.WriteLine( "Press Enter Key" );
      Console.Read();
    }

    /// <summary>
    /// Solve the Klee Minty cube linear programming problem with the primal simplex
    /// solver using different pivoting strategies.
    /// </summary>
    /// <param name="kleeMintyProblem">Klee Minty cube linear programming problem.</param>
    static void SolveKleeMintyProblemPrimal( LinearProgrammingProblem kleeMintyProblem )
    {
      
      // Create a primal simplex solver and maximize the objective function.
      var solver = new PrimalSimplexSolverORTools();
      solver.Solve( kleeMintyProblem, false );
      int partialPivotCount = solver.PivotCount;
      Console.WriteLine( "Primal Simplex Solver, Partial pivot result: " + solver.Result );
      Console.WriteLine( "Primal Simplex Solver, Optimal objecive value (1): " + solver.OptimalObjectiveFunctionValue);
      Console.WriteLine();
    }

    /// <summary>
    /// Solve the Klee Minty cube linear programming problem with the dual simplex
    /// solver using different pivoting strategies.
    /// </summary>
    /// <param name="kleeMintyProblem">Klee Minty cube linear programming problem.</param>
    static void SolveKleeMintyProblemDual( LinearProgrammingProblem kleeMintyProblem )
    {
      
      // Create a dual simplex solver and maximize the objective functions.
      var solver = new DualSimplexSolverORTools();
      solver.Solve( kleeMintyProblem, false );
      int steepestEdgePivotCount = solver.PivotCount;
      Console.WriteLine( "Dual Simplex Solver, Steepest Edge result: " + solver.Result );
      Console.WriteLine( "Primal Simplex Solver, Optimal objecive value (1): " + solver.OptimalObjectiveFunctionValue );
      Console.WriteLine();
    }

    /// <summary>
    /// Create the Klee Minty cube linear programming problem with the given dimension.
    /// </summary>
    /// <param name="n">The desired number of vertices.</param>
    /// <returns><c>LinearProgramming</c> object describing the Klee Minty
    /// cube linear programming problem.</returns>
    static LinearProgrammingProblem KleeMintyCube( int n )
    {
      var objectiveCoeff = new DoubleVector( n );
      objectiveCoeff[n - 1] = 1.0;
      double eps = 1.0 / 300.0;
      var problem = new LinearProgrammingProblem( objectiveCoeff );
      problem.AddBounds( 0, 0.0, 1.0 );
      for ( int i = 1; i < n; i++ )
      {
        // Lower bound constraint.
        var lbConstraintCoeffs = new DoubleVector( n );
        lbConstraintCoeffs[i] = 1.0;
        lbConstraintCoeffs[i - 1] = -eps;
        problem.AddLowerBoundConstraint( lbConstraintCoeffs, 0.0 );

        // Upper bound constraint
        var ubConstraintCoeffs = new DoubleVector( n );
        ubConstraintCoeffs[i] = 1.0;
        ubConstraintCoeffs[i - 1] = eps;
        problem.AddUpperBoundConstraint( ubConstraintCoeffs, 1.0 );
      }
      return problem;
    }
  }
}
